from typing import Optional, List, Mapping, Any
from time import sleep
from langchain.llms.base import LLM
from revChatGPT.V1 import Chatbot


class GPT4OpenAI(LLM):

    history_data: Optional[List] = []
    token : Optional[str]
    chatbot : Optional[Chatbot] = None
    call : int = 0
    conversation : Optional[str] = ""
    headless : bool = True
    __file__ = __file__
    model: str = "gpt-4"
    plugin_ids: List[str] = []

    @property
    def _llm_type(self) -> str:
        return "custom"

    def _call(self, prompt: str, stop: Optional[List[str]] = None) -> str:
        if self.chatbot is None:
            if self.token is None:
                raise ValueError("You need to specify the token, please check https://github.com/Erol444/gpt4-openai-api#how-to-get-the-access-token")

            self.chatbot = Chatbot({'access_token': self.token, 'model': self.model, 'plugin_ids': self.plugin_ids})

        response = ""
        for data in self.chatbot.ask(prompt):
            response = data["message"]

        # Add to history
        self.history_data.append({"prompt":prompt,"response":response})

        return response

    @property
    def _identifying_params(self) -> Mapping[str, Any]:
        """Get the identifying parameters."""
        return {"model": self.model, "token": self.token}



import os
from dotenv import load_dotenv
from gpt4_openai import GPT4OpenAI
load_dotenv()

# accessToken from https://chat.openai.com/api/auth/session
llm = GPT4OpenAI(token=os.environ["OPENAI_ACCESS_TOKEN"], model='gpt-4-browsing')
# ChatGPT will first browse the web for the name/age of her boyfriend, then return the answer
response = llm('What is the age difference between Dua Lipa and her boyfriend?')

print(response)

"""
Expected answer:
The age difference between Dua Lipa and her boyfriend Romain Gavras is 14 years. As of 2023, Romain Gavras is 41 years old, while Dua Lipa is 27 years old【5†source】.
"""


import os
from dotenv import load_dotenv
from gpt4_openai import GPT4OpenAI
load_dotenv()

# accessToken from https://chat.openai.com/api/auth/session
# Plugin IDs can be found here: https://github.com/acheong08/ChatGPT/blob/main/docs/plugins.json
llm = GPT4OpenAI(token=os.environ["OPENAI_ACCESS_TOKEN"],
                 model='gpt-4',
                 plugin_ids=['plugin-d1d6eb04-3375-40aa-940a-c2fc57ce0f51'] # Wolfram Alpha
                 )

# ChatGPT will use Wolfram Alpha plugin to calculate the equation
response = llm('Calculate the square root of 12345 to 10 decimal places')
print(response)

"""
Expected answer:

Calculating the square root of a number can be done using various methods, but for large numbers and precise decimal places, the best way is to use a calculator or programming tool.

Here's the square root of 12345 calculated to 10 decimal places:

√12345 = 111.1080555135
"""